package com.gic.haoban.manage.api.service;

import java.util.List;

import com.gic.api.base.commons.Page;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.DepartmentDTO;

public interface DepartmentApiService {

	List<DepartmentDTO> listByParentId(String parentId);
	/**
	 * 查门店/部门列表
	 * @param parentId
	 * @return
	 */
	List<DepartmentDTO> listByDepartmentIds(List<String> departmentIds,int storeFlag);
	/**
	 * 查wxEnterprise下所有门店
	 * @param wxEnterpriseId
	 * @return
	 */
	List<DepartmentDTO> listStoreByWxEnterpriseId(String wxEnterpriseId);
	
	/**
	 * 查wxEnterprise下所有门店(溢出门店)
	 * @param wxEnterpriseId
	 * @return
	 */
	Page<DepartmentDTO> pageFullStoreByWxEnterpriseId(
			String wxEnterpriseId, String search,List<String> storeId,int count,BasePageInfo pageInfo);
	/**
	 * 根据部门id查询部门
	 * @param departmentId
	 * @return
	 */
	DepartmentDTO selectById(String departmentId);
	/**
	 * 新增部门
	 * @param department
	 * @return
	 */
	HaobanResponse add(DepartmentDTO department);
	/**
	 * 修改部门
	 * @param department
	 * @return
	 */
	HaobanResponse edit(DepartmentDTO department);
	/**
	 * 删除部门
	 * @param departmentId
	 * @return
	 */
	HaobanResponse del(String departmentId);
	/**
	 * 根据关联id 获取部门
	 * @param departId
	 * @return
	 */
	DepartmentDTO selectByRelatedId(String departId);
	/**
	 * 放入回收站
	 * @param departmentId
	 * @return
	 */
	HaobanResponse recycle(String departmentId);
	
	/**
	 * 根据参数查询
	 * @param pageInfo
	 * @param wxEnterpriseId
	 * @param keyword
	 * @param storeFlag
	 * @param recycleFlag
	 * @return
	 */
	Page<DepartmentDTO> pageDepartmentByParams(BasePageInfo pageInfo, String wxEnterpriseId, String keyword,
			Integer storeFlag, Integer recycleFlag);
	/**
	 * 回收站恢复
	 * @param departmentId
	 */
	HaobanResponse repairRecycle(String departmentId,String parentId);
	/**
	 * 获取根节点
	 * @param wxEnterpriseId
	 * @return
	 */
	DepartmentDTO getRootByEnterpriseId(String wxEnterpriseId);
	/**
	 * gic同步
	 * @param fieldListString
	 */
	void syncGicStore(String fieldListString);
	/**
	 * gic同步部门
	 * @param fieldListString
	 */
	void syncGicDepartment(String fieldListString);
	/**
	 * 分页查询
	 * @param parentId
	 * @param pageInfo
	 * @return
	 */
	Page<DepartmentDTO> pageByParentId(String parentId,String keyword, BasePageInfo pageInfo);
	/**
	 * 根据链查询子部门和自己
	 * @param chainId
	 * @return
	 */
	List<DepartmentDTO> listByChainId(String chainId,String wxEnterpriseId);


	void initwxDepartment(String corpid, String suiteid, String wxEnterpriseId);
	/**
	 * 回收站的数量
	 * @param wxEnterpriseId
	 * @return
	 */
	Integer departmentRecycleCount(String wxEnterpriseId);
	/**
	 * 微信回调删除部门
	 * @param departmentId
	 * @return
	 */
	HaobanResponse wxCallBackDel(String departmentId);
	/**
	 * 微信回调放回收站
	 * @param departmentId
	 * @return
	 */
	HaobanResponse wxCallBackrecycle(String departmentId);
	/**
	 * 判断节点下面有没有子节点
	 * @param departmentId
	 * @return
	 */
	boolean hasSonNode(String departmentId);
	/**
	 * gic回调删除部门
	 * @param departmentId
	 */
	HaobanResponse gicDel(String departmentId);
	
}
