package com.gic.haoban.manage.web.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.hutool.core.collection.CollectionUtil;

import com.alibaba.csp.sentinel.util.StringUtil;
import com.alibaba.fastjson.JSONObject;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.api.dto.StaffDepartmentRelatedDTO;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.StaffDepartmentRelatedApiService;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.interceptor.WebInterceptor;
import com.gic.haoban.manage.web.vo.ClerkVo;
import com.gic.haoban.manage.web.vo.StoreVO;

@RestController
public class ClerkController extends WebBaseController{
    private static Logger logger= LoggerFactory.getLogger(WebInterceptor.class);

	@Autowired
	private ClerkService clerkService;
	@Autowired
	private StaffApiService staffApiService;
	@Autowired
	private StaffDepartmentRelatedApiService staffDepartmentRelatedApiService;
	@Autowired
	private StoreService storeService;
	@Autowired
	private DepartmentApiService departmentApiService;
	
	//导购列表
	@RequestMapping("/clerk-list")
	public HaobanResponse clerkList(String storeId,String bindFlag,String departmentId) {
		List<StaffDepartmentRelatedDTO> list = staffDepartmentRelatedApiService.listByDepartmentId(departmentId);
		List<ClerkVo> result  = new ArrayList<ClerkVo>();
//		if(CollectionUtil.isNotEmpty(list)){
//			if("1".equals(bindFlag)){
//				List<StaffDepartmentRelatedDTO> bindList = list.stream().filter(s->s.getClerkCode() != null).collect(Collectors.toList());
//				if(CollectionUtil.isNotEmpty(bindList)){
//					result = EntityUtil.changeEntityListByJSON(ClerkVo.class, bindList);
//				}
//			}else{
//
//				List<StaffDepartmentRelatedDTO> unBindList = list.stream().filter(s->s.getClerkCode() == null).collect(Collectors.toList());
//				if(CollectionUtil.isNotEmpty(unBindList)){
//					result = EntityUtil.changeEntityListByJSON(ClerkVo.class, unBindList);
//				}
//			}
//		}
//		return resultResponse(HaoBanErrCode.ERR_1,result);
		if(StringUtils.isAnyBlank(storeId,bindFlag,departmentId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		//绑定的clerk
		if("1".equals(bindFlag)){
			// 调陶接口获取详情数据
			if(CollectionUtil.isNotEmpty(list)){
				List<String> clerkCodes = list.stream().filter(s->StringUtils.isNotBlank(s.getClerkCode())).map(s->s.getClerkCode()).collect(Collectors.toList());
				logger.info("clerkCodes =============>为{}",JSONObject.toJSONString(clerkCodes));
				if(CollectionUtil.isNotEmpty(clerkCodes)){
					List<ClerkDTO> clerkList = clerkService.listClerk(storeId, clerkCodes, true);
					result = EntityUtil.changeEntityListByJSON(ClerkVo.class, clerkList);
				}
			}
		}else{
			//未绑定的clerk
			// 调陶接口获取详情数据
			if(CollectionUtil.isNotEmpty(list)){
				List<String> clerkCodes = list.stream().filter(s->StringUtils.isNotBlank(s.getClerkCode())).map(s->s.getClerkCode()).collect(Collectors.toList());
				if(CollectionUtil.isNotEmpty(clerkCodes)){
					List<ClerkDTO> clerkList = clerkService.listClerk(storeId, clerkCodes, false);
					result = EntityUtil.changeEntityListByJSON(ClerkVo.class, clerkList);
				}
			}else{
				List<String> clerkCodes = new ArrayList();
				clerkCodes.add("xxxXXXX");
				List<ClerkDTO> clerkList = clerkService.listClerk(storeId, clerkCodes, false);
				result = EntityUtil.changeEntityListByJSON(ClerkVo.class, clerkList);
			}
		}
		return resultResponse(HaoBanErrCode.ERR_1,result);
		
	}
	//选择成员列表
	@RequestMapping("/staff-list")
	public HaobanResponse staffList(String departmentId) {
		if(StringUtils.isAnyBlank(departmentId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		List<StaffDepartmentRelatedDTO> list = staffDepartmentRelatedApiService.listByDepartmentId(departmentId);
		Map<String,StaffDepartmentRelatedDTO> relatedMap = com.gic.commons.util.CollectionUtil.toMap(list, "staffId");
		List<String> staffIds = list.stream().map(s->s.getStaffId()).collect(Collectors.toList());
		List<StaffDTO>  resultlist= staffApiService.listByIds(staffIds);
		for(StaffDTO dto :resultlist){
			if(relatedMap.get(dto.getStaffId()) != null){
				dto.setStaffDepartmentRelatedId(relatedMap.get(dto.getStaffId()).getStaffDepartmentRelatedId());
				if(relatedMap.get(dto.getStaffId()).getClerkCode()!=null){
					dto.setBindFlag(1);
					dto.setClerkCode(relatedMap.get(dto.getStaffId()).getClerkCode());
				}else{
					dto.setBindFlag(0);
				}
			}else{
				dto.setBindFlag(0);
			}
		}
		
		//查找店长clerkType
		DepartmentDTO departmentDTO = departmentApiService.selectById(departmentId);
		String storeId = departmentDTO.getRelatedId();
		if(StringUtils.isEmpty(storeId)){
		}else{
			String storeClerkCode = "";
			StoreDTO storeDTO = storeService.getStore(storeId);
			if(storeDTO!=null){
				String clerkId = storeDTO.getClerkId();
				ClerkDTO clerkDTO = clerkService.getClerkByClerkId(clerkId);
				if(clerkDTO!=null){
					storeClerkCode = clerkDTO.getClerkCode();
				}
			}
			if(StringUtil.isNotBlank(storeClerkCode)){
				for(StaffDTO dto :resultlist){
					if(storeClerkCode.equals(dto.getClerkCode())){
						dto.setClerkType(1);
					}
				}
			}
		}
		
		logger.info("result   ClerkCode =================>{}",JSONObject.toJSONString(resultlist));
		return resultResponse(HaoBanErrCode.ERR_1,resultlist);
		
	}
	//成员绑定
	@RequestMapping("/staff-bind")
	public HaobanResponse staffBind(String departmentId,String fromClerkCode,String fromStaffDepartmentRelatedId) {
		if(StringUtils.isAnyBlank(departmentId,fromClerkCode,fromStaffDepartmentRelatedId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		//将指定人绑定clerkCode
		StaffDepartmentRelatedDTO dto = new StaffDepartmentRelatedDTO();
		dto.setStaffDepartmentRelatedId(fromStaffDepartmentRelatedId);
		dto.setClerkCode(fromClerkCode);
		dto.setUpdateTime(new Date());
		staffDepartmentRelatedApiService.updateById(dto);
		
		//更新gic数据
		DepartmentDTO departmentDTO = departmentApiService.selectById(departmentId);
		String storeId = departmentDTO.getRelatedId();
		List<String> clerkCodes = new ArrayList<>();
		clerkCodes.add(fromClerkCode);
		List<ClerkDTO> list = clerkService.listClerk(storeId, clerkCodes, true);
		StaffDepartmentRelatedDTO staffDepartmentRelatedDTO = staffDepartmentRelatedApiService.getByStaffDepartmentRelatedId(fromStaffDepartmentRelatedId);
		StaffDTO staffDTO = staffApiService.selectById(staffDepartmentRelatedDTO.getStaffId());
		for(ClerkDTO clerkDTO :list){
			clerkDTO.setPhoneNumber(staffDTO.getPhoneNumber());
			clerkDTO.setClerkName(staffDTO.getStaffName());
			clerkService.updateClerk(clerkDTO);
		}
		
		
		return resultResponse(HaoBanErrCode.ERR_1);
	}
	//成员解除绑定
	@RequestMapping("/clerk-single-unbind")
	public HaobanResponse clerkSingleUnbind(String fromClerkCode,String departmentId) {
		if(StringUtils.isAnyBlank(fromClerkCode)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		//将自己的clerkCode置空
		StaffDepartmentRelatedDTO one = staffDepartmentRelatedApiService.getOneByClerkCodeAndDepartmentId(fromClerkCode, departmentId);
		if(one != null){
			//存在，则先置空
			one.setClerkCode(null);
			one.setUpdateTime(new Date());
			staffDepartmentRelatedApiService.deleteCode(one);
		}
		//更新gic数据
		DepartmentDTO departmentDTO = departmentApiService.selectById(departmentId);
		String storeId = departmentDTO.getRelatedId();
		List<String> clerkCodes = new ArrayList<>();
		clerkCodes.add(fromClerkCode);
		List<ClerkDTO> list = clerkService.listClerk(storeId, clerkCodes, true);
		for(ClerkDTO clerkDTO :list){
			clerkDTO.setPhoneNumber("--");
			clerkService.updateClerk(clerkDTO);
		}
		
		return resultResponse(HaoBanErrCode.ERR_1);
	}
	//成员换绑定
	@RequestMapping("/clerk-unbind")
	public HaobanResponse clerkUnbind(String departmentId,String fromClerkCode,String toStaffId,String manager) {
		if(StringUtils.isAnyBlank(fromClerkCode,toStaffId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		StaffDepartmentRelatedDTO staffDepartmentRelatedDTO = staffDepartmentRelatedApiService.getOneByStaffIdAndDepartmentId(toStaffId, departmentId);
		String oldClerkCode = staffDepartmentRelatedDTO.getClerkCode();
		if("1".equals(manager)){
			//店长操作
			//第一个点击的人的clerkCode置空
			StaffDepartmentRelatedDTO one = staffDepartmentRelatedApiService.getOneByClerkCodeAndDepartmentId(fromClerkCode,departmentId);
			if(one != null){
				//存在，则先置空
				one.setClerkCode(null);
				one.setUpdateTime(new Date());
				staffDepartmentRelatedApiService.deleteCode(one);
			}
			StaffDepartmentRelatedDTO dto = staffDepartmentRelatedApiService.getOneByStaffIdAndDepartmentId(toStaffId, departmentId);
			if(dto == null){
				return resultResponse(HaoBanErrCode.ERR_400017);
			}
				//存在，则先置空
			dto.setClerkCode(fromClerkCode);
			dto.setUpdateTime(new Date());
			staffDepartmentRelatedApiService.updateById(dto);
			//更新gic数据
			DepartmentDTO departmentDTO = departmentApiService.selectById(departmentId);
			String storeId = departmentDTO.getRelatedId();
			List<String> clerkCodes = new ArrayList<>();
			clerkCodes.add(fromClerkCode);
			List<ClerkDTO> list = clerkService.listClerk(storeId, clerkCodes, true);
			StaffDTO staffDTO = staffApiService.selectById(toStaffId);
			for(ClerkDTO clerkDTO :list){
				clerkDTO.setPhoneNumber(staffDTO.getPhoneNumber());
				clerkDTO.setClerkName(staffDTO.getStaffName());
				clerkService.updateClerk(clerkDTO);
			}
			//清空gic原来的手机号
			List<String> oldClerkCodes = new ArrayList<>();
			oldClerkCodes.add(oldClerkCode);
			List<ClerkDTO> oldList = clerkService.listClerk(storeId, oldClerkCodes, true);
			for(ClerkDTO clerkDTO :oldList){
				clerkDTO.setPhoneNumber("--");
				clerkService.updateClerk(clerkDTO);
			}

		}else{
			//把别人的的clerkCode置空
			logger.info("000000000000");
			StaffDepartmentRelatedDTO one = staffDepartmentRelatedApiService.getOneByClerkCodeAndDepartmentId(fromClerkCode,departmentId);
			if(one != null){
				//存在，则先置空
				logger.info("111111111111111");
				one.setClerkCode(null);
				one.setUpdateTime(new Date());
				staffDepartmentRelatedApiService.deleteCode(one);
			}
			logger.info("222222222222222");
			//别人的clerkCOde换到自己上
			StaffDepartmentRelatedDTO dto = staffDepartmentRelatedApiService.getOneByStaffIdAndDepartmentId(toStaffId, departmentId);
			if(dto == null){
				return resultResponse(HaoBanErrCode.ERR_400017);
			}
			logger.info("3333333333333");
			dto.setClerkCode(fromClerkCode);
			dto.setUpdateTime(new Date());
			staffDepartmentRelatedApiService.updateById(dto);
			//更新gic数据
			DepartmentDTO departmentDTO = departmentApiService.selectById(departmentId);
			String storeId = departmentDTO.getRelatedId();
			List<String> clerkCodes = new ArrayList<>();
			clerkCodes.add(fromClerkCode);
			List<ClerkDTO> list = clerkService.listClerk(storeId, clerkCodes, true);
			StaffDTO staffDTO = staffApiService.selectById(toStaffId);
			logger.info("444444444444");
			for(ClerkDTO clerkDTO :list){
				clerkDTO.setPhoneNumber(staffDTO.getPhoneNumber());
				clerkDTO.setClerkName(staffDTO.getStaffName());
				clerkService.updateClerk(clerkDTO);
			}
			
			//清空gic原来的手机号
			List<String> oldClerkCodes = new ArrayList<>();
			oldClerkCodes.add(oldClerkCode);
			logger.info("oldClerkCode====================>{}",oldClerkCode);
			List<ClerkDTO> oldList = clerkService.listClerk(storeId, oldClerkCodes, true);
			logger.info("oldList====================>{}",JSONObject.toJSONString(oldList));
			for(ClerkDTO clerkDTO :oldList){
				clerkDTO.setPhoneNumber("--");
				clerkDTO.setClerkName("--");
				clerkService.updateClerk(clerkDTO);
			}
		}
		
		return resultResponse(HaoBanErrCode.ERR_1);
	}
	//店长转让
	@RequestMapping("/clerk-manager-tranfer")
	public HaobanResponse clerkManagerTranfer(String departmentId,String fromClerkCode,String toStaffDepartmentRelatedId) {
		if(StringUtils.isAnyBlank(fromClerkCode,toStaffDepartmentRelatedId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		StaffDepartmentRelatedDTO one = staffDepartmentRelatedApiService.getOneByClerkCodeAndDepartmentId(fromClerkCode,departmentId);
		if(one != null){
			//存在，则先置空
			one.setClerkCode(null);
			one.setUpdateTime(new Date());
			staffDepartmentRelatedApiService.deleteCode(one);
		}
		//将其他人更新为店长
		StaffDepartmentRelatedDTO dto = new StaffDepartmentRelatedDTO();
		dto.setStaffDepartmentRelatedId(toStaffDepartmentRelatedId);
		dto.setClerkCode(fromClerkCode);
		dto.setUpdateTime(new Date());
		staffDepartmentRelatedApiService.updateById(dto);
		
		//更新gic数据
		DepartmentDTO departmentDTO = departmentApiService.selectById(departmentId);
		String storeId = departmentDTO.getRelatedId();
		List<String> clerkCodes = new ArrayList<>();
		clerkCodes.add(fromClerkCode);
		List<ClerkDTO> list = clerkService.listClerk(storeId, clerkCodes, true);
		StaffDepartmentRelatedDTO staffDepartmentRelatedDTO = staffDepartmentRelatedApiService.getByStaffDepartmentRelatedId(toStaffDepartmentRelatedId);
		StaffDTO staffDTO = staffApiService.selectById(staffDepartmentRelatedDTO.getStaffId());
		for(ClerkDTO clerkDTO :list){
			clerkDTO.setPhoneNumber(staffDTO.getPhoneNumber());
			clerkDTO.setClerkName(staffDTO.getStaffName());
			clerkService.updateClerk(clerkDTO);
		}
		
		return resultResponse(HaoBanErrCode.ERR_1);
	}
}
